from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.auto_generate_column_name_apply_prompt import AutoGenerateColumnNameApplyPrompt
from ...models.auto_generate_column_name_data_lookup import AutoGenerateColumnNameDataLookup
from ...models.auto_generate_column_name_validated_prompt import AutoGenerateColumnNameValidatedPrompt
from ...models.http_validation_error import HTTPValidationError
from ...models.response_with_metadata_schemastr import ResponseWithMetadataSchemastr
from ...types import Response


def _get_kwargs(
    project_id: UUID,
    *,
    body: Union[
        "AutoGenerateColumnNameApplyPrompt", "AutoGenerateColumnNameDataLookup", "AutoGenerateColumnNameValidatedPrompt"
    ],
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": f"/custom-fields/{project_id}/column-names",
    }

    _body: dict[str, Any]
    if isinstance(body, AutoGenerateColumnNameDataLookup):
        _body = body.to_dict()
    elif isinstance(body, AutoGenerateColumnNameApplyPrompt):
        _body = body.to_dict()
    else:
        _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]:
    if response.status_code == 200:
        response_200 = ResponseWithMetadataSchemastr.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        "AutoGenerateColumnNameApplyPrompt", "AutoGenerateColumnNameDataLookup", "AutoGenerateColumnNameValidatedPrompt"
    ],
) -> Response[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]:
    """Auto Generate Column Name

     Generate an automatic name for a custom column based on the provided configuration.

    Args:
    - **input_data**: Configuration of the custom column to creat name for depending on the required
    type in next formats:
        - AutoGenerateColumnNameDataLookup
        - AutoGenerateColumnNameApplyPrompt
        - AutoGenerateColumnNameValidatedPrompt

    Returns:
    - **str**: Generated name of a column.

    Raises:
    - **401 Unauthorized (CredentialException)**: Raised if the user is not found in Firebase.
    - **401 Unauthorized (PermissionException)**: Raised if the API key is invalid, expired or no valid
    key or bearer token was provided.
    - **403 Forbidden (UserNotAuthenticated)**: Raised if user ID could not be extracted from the token.
    - **404 Not Found (OpenRouterModelException)**: Raised if the requested model is not allowed or
    available.
    - **500 Internal Server Error (ValueError)**: Raised if input formatting or response validation
    fails.
    - **500 Internal Server Error (RuntimeError)**: Raised if any unexpected error occurs during the LLM
    request.

    Args:
        project_id (UUID):
        body (Union['AutoGenerateColumnNameApplyPrompt', 'AutoGenerateColumnNameDataLookup',
            'AutoGenerateColumnNameValidatedPrompt']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        "AutoGenerateColumnNameApplyPrompt", "AutoGenerateColumnNameDataLookup", "AutoGenerateColumnNameValidatedPrompt"
    ],
) -> Optional[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]:
    """Auto Generate Column Name

     Generate an automatic name for a custom column based on the provided configuration.

    Args:
    - **input_data**: Configuration of the custom column to creat name for depending on the required
    type in next formats:
        - AutoGenerateColumnNameDataLookup
        - AutoGenerateColumnNameApplyPrompt
        - AutoGenerateColumnNameValidatedPrompt

    Returns:
    - **str**: Generated name of a column.

    Raises:
    - **401 Unauthorized (CredentialException)**: Raised if the user is not found in Firebase.
    - **401 Unauthorized (PermissionException)**: Raised if the API key is invalid, expired or no valid
    key or bearer token was provided.
    - **403 Forbidden (UserNotAuthenticated)**: Raised if user ID could not be extracted from the token.
    - **404 Not Found (OpenRouterModelException)**: Raised if the requested model is not allowed or
    available.
    - **500 Internal Server Error (ValueError)**: Raised if input formatting or response validation
    fails.
    - **500 Internal Server Error (RuntimeError)**: Raised if any unexpected error occurs during the LLM
    request.

    Args:
        project_id (UUID):
        body (Union['AutoGenerateColumnNameApplyPrompt', 'AutoGenerateColumnNameDataLookup',
            'AutoGenerateColumnNameValidatedPrompt']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, ResponseWithMetadataSchemastr]
    """

    return sync_detailed(
        project_id=project_id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    project_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        "AutoGenerateColumnNameApplyPrompt", "AutoGenerateColumnNameDataLookup", "AutoGenerateColumnNameValidatedPrompt"
    ],
) -> Response[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]:
    """Auto Generate Column Name

     Generate an automatic name for a custom column based on the provided configuration.

    Args:
    - **input_data**: Configuration of the custom column to creat name for depending on the required
    type in next formats:
        - AutoGenerateColumnNameDataLookup
        - AutoGenerateColumnNameApplyPrompt
        - AutoGenerateColumnNameValidatedPrompt

    Returns:
    - **str**: Generated name of a column.

    Raises:
    - **401 Unauthorized (CredentialException)**: Raised if the user is not found in Firebase.
    - **401 Unauthorized (PermissionException)**: Raised if the API key is invalid, expired or no valid
    key or bearer token was provided.
    - **403 Forbidden (UserNotAuthenticated)**: Raised if user ID could not be extracted from the token.
    - **404 Not Found (OpenRouterModelException)**: Raised if the requested model is not allowed or
    available.
    - **500 Internal Server Error (ValueError)**: Raised if input formatting or response validation
    fails.
    - **500 Internal Server Error (RuntimeError)**: Raised if any unexpected error occurs during the LLM
    request.

    Args:
        project_id (UUID):
        body (Union['AutoGenerateColumnNameApplyPrompt', 'AutoGenerateColumnNameDataLookup',
            'AutoGenerateColumnNameValidatedPrompt']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        "AutoGenerateColumnNameApplyPrompt", "AutoGenerateColumnNameDataLookup", "AutoGenerateColumnNameValidatedPrompt"
    ],
) -> Optional[Union[HTTPValidationError, ResponseWithMetadataSchemastr]]:
    """Auto Generate Column Name

     Generate an automatic name for a custom column based on the provided configuration.

    Args:
    - **input_data**: Configuration of the custom column to creat name for depending on the required
    type in next formats:
        - AutoGenerateColumnNameDataLookup
        - AutoGenerateColumnNameApplyPrompt
        - AutoGenerateColumnNameValidatedPrompt

    Returns:
    - **str**: Generated name of a column.

    Raises:
    - **401 Unauthorized (CredentialException)**: Raised if the user is not found in Firebase.
    - **401 Unauthorized (PermissionException)**: Raised if the API key is invalid, expired or no valid
    key or bearer token was provided.
    - **403 Forbidden (UserNotAuthenticated)**: Raised if user ID could not be extracted from the token.
    - **404 Not Found (OpenRouterModelException)**: Raised if the requested model is not allowed or
    available.
    - **500 Internal Server Error (ValueError)**: Raised if input formatting or response validation
    fails.
    - **500 Internal Server Error (RuntimeError)**: Raised if any unexpected error occurs during the LLM
    request.

    Args:
        project_id (UUID):
        body (Union['AutoGenerateColumnNameApplyPrompt', 'AutoGenerateColumnNameDataLookup',
            'AutoGenerateColumnNameValidatedPrompt']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, ResponseWithMetadataSchemastr]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            client=client,
            body=body,
        )
    ).parsed
